unit ExportXmlUnit;

interface

uses
  Classes,Dialogs,SysUtils,Graphics,XMLDoc,XMLIntf,EncdDecd,CAD;

function TColor2RGB(const Color: TColor):string;
function SaveToXML(MyCAD:TMyCAD;FileName: string): Boolean;
function SaveToDoc(MyCAD:TMyCAD;document:TXmlDocument):Boolean;

procedure SaveShapeToDoc(MyShape:TMyShape;document:TXmlDocument;shape:IXMLNode);
procedure SaveShapeLineToDoc(MyShape:TMyShape;document:TXmlDocument;shape:IXMLNode);
procedure SaveShapeRuleLineToDoc(MyShape:TMyShape;document:TXmlDocument;shape:IXMLNode);
procedure SaveShapeImageToDoc(MyShape:TMyShape;document:TXmlDocument;shape:IXMLNode);
procedure SaveShapeTextToDoc(MyShape:TMyShape;document:TXmlDocument;shape:IXMLNode);
procedure SaveShapeLinkPointToDoc(MyShape:TMyShape;document:TXmlDocument;shape:IXMLNode);
procedure SaveShapeElliArcToDoc(MyShape:TMyShape;document:TXmlDocument;shape:IXMLNode);


implementation

function TColor2RGB(const Color: TColor):string;
var
  R,G,B:Byte;
begin
  R := Color and $FF;
  G := (Color shr 8) and $FF;
  B := (Color shr 16) and $FF;
  result := '#'+IntToHex(R,2)+IntToHex(G,2)+IntToHex(B,2);
end;

function SaveToXML(MyCAD:TMyCAD;FileName: string): Boolean;
var
  document:TXMLDocument;
begin
  try
    document := TXMLDocument.Create(nil);
    result := SaveToDoc(MyCAD,document);
    if result then
      document.SaveToFile(FileName);
  except
    ShowMessage('Save to XML file failed!');
  end;
end;

function SaveToDoc(MyCAD:TMyCAD;document: TXmlDocument): Boolean;
var
  rNode:IXMLNode;
  pNode,cNode,cNode2:IXMLNode;

  mem:TMemoryStream;
  strStream:TStringStream;

  i:integer;
begin
  result := false;
  try
    document.Active := true;
    document.Version := '1.0';
    document.Encoding := 'GB2312';

    //Ŀ¼
    rNode := document.CreateNode('UltraGraphics');
    document.DocumentElement := rNode;
    
    //Name
    pNode := document.CreateNode('Name');
    pNode.Text := 'TCAD for WEB';
    document.DocumentElement.ChildNodes.Add(pNode);

    //Version
    pNode := document.CreateNode('Version');
    pNode.Text := '1.0';
    document.DocumentElement.ChildNodes.Add(pNode);

    //Date
    pNode := document.CreateNode('Date');
    pNode.Text := '2009-01-01';
    document.DocumentElement.ChildNodes.Add(pNode);

    //MyCAD
    pNode := document.CreateNode('MyCAD');
    document.DocumentElement.ChildNodes.Add(pNode);

    //Alpha - .net
    {
    cNode := document.CreateNode('Alpha');
    pNode.ChildNodes.Add(cNode);
    }

    //ArrowAnlge
    cNode := document.CreateNode('ArrowAngle');
    cNode.Text := IntToStr(MyCAD.ArrowAngle);
    pNode.ChildNodes.Add(cNode);

    //ArrowLength
    cNode := document.CreateNode('ArrowLength');
    cNode.Text := IntToStr(MyCAD.ArrowLength);
    pNode.ChildNodes.Add(cNode);

    //ArrowOffset
    cNode := document.CreateNode('ArrowOffset');
    cNode.Text := IntToStr(MyCAD.ArrowOffset);
    pNode.ChildNodes.Add(cNode);

    //ArrowStyle
    cNode := document.CreateNode('ArrowStyle');
    cNode.Text := IntToStr(Ord(MyCAD.ArrowStyle));
    pNode.ChildNodes.Add(cNode);
    
    //BackgroundBitmap
    if Assigned(MyCAD.BkBitmap) and
      (MyCAD.BkBitmap.Width >0) and
      (MyCAD.BkBitmap.Height >0) then
    begin

      strStream := TStringStream.Create('');
      mem := TMemoryStream.Create;
      MyCAD.BkBitmap.SaveToStream(mem);
      mem.Position := 0;
      EncodeStream(mem,strStream);

      cNode := document.CreateNode('BackgroundBitmap');
      cNode.Text := strStream.DataString;
      pNode.ChildNodes.Add(cNode);

      strStream.Destroy;
      mem.Destroy;

    end;

    //BackgroundBitmapMode
    cNode := document.CreateNode('BackgroundBitmapMode');
    cNode.Text := IntToStr(Ord(MyCAD.BkBitmapMode));
    pNode.ChildNodes.Add(cNode);

    //BackgroundColor
    cNode := document.CreateNode('BackgroundColor');
    cNode.Text := TColor2RGB(MyCAD.ColorOfBackGround);
    pNode.ChildNodes.Add(cNode);

    //Brush
    cNode := document.CreateNode('Brush');
    cNode.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
    cNode.SetAttribute('ClassName','System.Drawing.SolidBrush');
    pNode.ChildNodes.Add(cNode);

    //Color
    cNode2 := document.CreateNode('Color');
    cNode2.Text := TColor2RGB(MyCAD.Brush.Color);
    cNode.ChildNodes.Add(cNode2);

    //BrushShow - .net
    {
    cNode := document.CreateNode('BrushShow');
    pNode.ChildNodes.Add(cNode);
    }

    //CrossLine
    cNode := document.CreateNode('CrossLine');
    if MyCAD.CrossLine then
      cNode.Text := 'True'
    else
      cNode.Text := 'False';
    pNode.ChildNodes.Add(cNode);

    //CurrentLayerId
    cNode := document.CreateNode('CurrentLayerId');
    cNode.Text := IntToStr(MyCAD.CurrentLayerId);
    pNode.ChildNodes.Add(cNode);

    //DragMode
    cNode := document.CreateNode('DragMode');
    cNode.Text := IntToStr(Ord(MyCAD.DragMode));
    pNode.ChildNodes.Add(cNode);

    //DragType
    cNode := document.CreateNode('DrawType');
    cNode.Text := IntToStr(Ord(MyCAD.ShapeTool));
    pNode.ChildNodes.Add(cNode);

    //FontName
    cNode := document.CreateNode('FontName');
    cNode.Text := MyCAD.Font.Name;
    pNode.ChildNodes.Add(cNode);

    //FontSize
    cNode := document.CreateNode('FontSize');
    cNode.Text := IntToStr(MyCAD.Font.Size);
    pNode.ChildNodes.Add(cNode);

    {
    //FontStyle
    cNode := document.CreateNode('FontStyle');
    pNode.ChildNodes.Add(cNode);
    //FontStrikeout
    cNode := document.CreateNode('Strikeout');
    pNode.ChildNodes.Add(cNode);
    //FontUnderline
    cNode := document.CreateNode('Underline');
    pNode.ChildNodes.Add(cNode);
    }

    //GridColor
    cNode := document.CreateNode('GridColor');
    cNode.Text := TColor2RGB(MyCAD.GridOperation.GridColor);
    pNode.ChildNodes.Add(cNode);

    //GridHeight
    cNode := document.CreateNode('GridHeight');
    cNode.Text := FloatToStr(MyCAD.GridOperation.GridHeight);
    pNode.ChildNodes.Add(cNode);

    //gridPenSize
    cNode := document.CreateNode('gridPenSize');
    cNode.Text := IntToStr(MyCAD.GridOperation.GridPenSize);
    pNode.ChildNodes.Add(cNode);

    //GridShow
    cNode := document.CreateNode('GridShow');
    if MyCAD.GridOperation.GridShow then
      cNode.Text := 'True'
    else
      cNode.Text := 'False';
    pNode.ChildNodes.Add(cNode);

    //GridType
    cNode := document.CreateNode('GridType');
    cNode.Text := IntToStr(ord(MyCAD.GridOperation.GridType));
    pNode.ChildNodes.Add(cNode);

    //GridWidth
    cNode := document.CreateNode('GridWidth');
    cNode.Text := FloatToStr(MyCAD.GridOperation.GridWidth);
    pNode.ChildNodes.Add(cNode);

    //HotColor
    cNode := document.CreateNode('HotColor');
    cNode.Text := TColor2RGB(MyCAD.ColorOfHot);
    pNode.ChildNodes.Add(cNode);

    //HotShow
    cNode := document.CreateNode('HotShow');
    if MyCAD.HotShow then
      cNode.Text := 'True'
    else
      cNode.Text := 'False';
    pNode.ChildNodes.Add(cNode);

    //HotSize
    cNode := document.CreateNode('HotSize');
    cNode.Text := IntToStr(MyCAD.HotSize);
    pNode.ChildNodes.Add(cNode);

    //LinklineAroundShape
    cNode := document.CreateNode('LinklineAroundShape');
    if MyCAD.LinklineAroundShape then
      cNode.Text := 'True'
    else
      cNode.Text := 'False';
    pNode.ChildNodes.Add(cNode);

    //LinklineDrawStyle
    cNode := document.CreateNode('LinklineDrawStyle');
    cNode.Text := IntToStr(Ord(MyCAD.LinkLineDrawStyle));
    pNode.ChildNodes.Add(cNode);

    //OperateAllLayer
    cNode := document.CreateNode('OperateAllLayer');
    if MyCAD.OperateAllLayer then
      cNode.Text := 'True'
    else
      cNode.Text := 'False';
    pNode.ChildNodes.Add(cNode);

    //PageFoot
    cNode := document.CreateNode('PageFoot');
    cNode.Text := MyCAD.PageFoot;
    pNode.ChildNodes.Add(cNode);

    //PageFootAlignment
    cNode := document.CreateNode('PageFootAlignment');
    cNode.Text := IntToStr(Ord(MyCAD.PageFootAlignment));
    pNode.ChildNodes.Add(cNode);

    //PageFootToBottom
    cNode := document.CreateNode('PageFootToBottom');
    cNode.Text := IntToStr(MyCAD.PageFootToBottom);
    pNode.ChildNodes.Add(cNode);

    //PageHead
    cNode := document.CreateNode('PageHead');
    cNode.Text := MyCAD.PageHead;
    pNode.ChildNodes.Add(cNode);

    //PageHeadAlignment
    cNode := document.CreateNode('PageHeadAlignment');
    cNode.Text := IntToStr(Ord(MyCAD.PageHeadAlignment));
    pNode.ChildNodes.Add(cNode);

    //PageHeadToTop
    cNode := document.CreateNode('PageHeadToTop');
    cNode.Text := IntToStr(MyCAD.PageHeadToTop);
    pNode.ChildNodes.Add(cNode);

    //PageHeight
    cNode := document.CreateNode('PageHeight');
    cNode.Text := IntToStr(MyCAD.PageHeight);
    pNode.ChildNodes.Add(cNode);

    //PageOrientation
    cNode := document.CreateNode('PageOrientation');
    cNode.Text := IntToStr(Ord(MyCAD.PageOrientation));
    pNode.ChildNodes.Add(cNode);

    //PageStyle
    cNode := document.CreateNode('PageStyle');
    cNode.Text := IntToStr(Ord(MyCAD.PageStyle));
    pNode.ChildNodes.Add(cNode);

    //PageWidth
    cNode := document.CreateNode('PageWidth');
    cNode.Text := IntToStr(MyCAD.PageWidth);
    pNode.ChildNodes.Add(cNode);

    //PenColor
    cNode := document.CreateNode('PenColor');
    cNode.Text := TColor2RGB(MyCAD.Pen.Color);
    pNode.ChildNodes.Add(cNode);

    //PenStyle
    cNode := document.CreateNode('PenStyle');
    cNode.Text := IntToStr(Ord(MyCAD.Pen.Style));
    pNode.ChildNodes.Add(cNode);

    //PenWidth
    cNode := document.CreateNode('PenWidth');
    cNode.Text := IntToStr(MyCAD.Pen.Width);
    pNode.ChildNodes.Add(cNode);

    //PrintBackground
    cNode := document.CreateNode('PrintBackground');
    if MyCAD.PrintBackground then
      cNode.Text := 'True'
    else
      cNode.Text := 'False';
    pNode.ChildNodes.Add(cNode);

    //PrintBorder
    cNode := document.CreateNode('PrintBorder');
    if MyCAD.PrintABorder then
      cNode.Text := 'True'
    else
      cNode.Text := 'False';
    pNode.ChildNodes.Add(cNode);

    //PrintBorderToBottom
    cNode := document.CreateNode('PrintBorderToBottom');
    cNode.Text := IntToStr(MyCAD.PrintABordertoBottom);
    pNode.ChildNodes.Add(cNode);

    //PrintBorderToLeft
    cNode := document.CreateNode('PrintBorderToLeft');
    cNode.Text := IntToStr(MyCAD.PrintABordertoLeft);
    pNode.ChildNodes.Add(cNode);

    //PrintBorderToRight
    cNode := document.CreateNode('PrintBorderToRight');
    cNode.Text := IntToStr(MyCAD.PrintABordertoRight);
    pNode.ChildNodes.Add(cNode);

    //PrintBorderToTop
    cNode := document.CreateNode('PrintBorderToTop');
    cNode.Text := IntToStr(MyCAD.PrintABordertoTop);
    pNode.ChildNodes.Add(cNode);

    //PreZoom
    cNode := document.CreateNode('PreZoom');
    cNode.Text := FloatToStr(MyCAD.PreZoom);
    pNode.ChildNodes.Add(cNode);

    //Ratio
    cNode := document.CreateNode('Ratio');
    cNode.Text := FloatToStr(MyCAD.Ratio);
    pNode.ChildNodes.Add(cNode);

    //ResizeEnable
    cNode := document.CreateNode('ResizeEnable');
    if MyCAD.ResizeEnable then
      cNode.Text := 'True'
    else
      cNode.Text := 'False';
    pNode.ChildNodes.Add(cNode);

    //ReturnToSelecting
    cNode := document.CreateNode('ReturnToSelecting');
    if MyCAD.ReturnToSelecting then
      cNode.Text := 'True'
    else
      cNode.Text := 'False';
    pNode.ChildNodes.Add(cNode);

    //RotateConstraintDegree
    cNode := document.CreateNode('RotateConstraintDegree');
    cNode.Text := IntToStr(MyCAD.RotateConstraintDegree);
    pNode.ChildNodes.Add(cNode);

    //RotateEnable
    cNode := document.CreateNode('RotateEnable');
    if MyCAD.RotateEnable then
      cNode.Text := 'True'
    else
      cNode.Text := 'False';
    pNode.ChildNodes.Add(cNode);

    //ShowHotLink
    cNode := document.CreateNode('ShowHotLink');
    if MyCAD.ShowHotLink then
      cNode.Text := 'True'
    else
      cNode.Text := 'False';
    pNode.ChildNodes.Add(cNode);

    //Snap
    cNode := document.CreateNode('Snap');
    if MyCAD.Snap then
      cNode.Text := 'True'
    else
      cNode.Text := 'False';
    pNode.ChildNodes.Add(cNode);

    //SnapPixel
    cNode := document.CreateNode('SnapPixel');
    cNode.Text := IntToStr(MyCAD.SnapPixels);
    pNode.ChildNodes.Add(cNode);

    //SnapShape
    cNode := document.CreateNode('SnapShape');
    if MyCAD.SnapShape then
      cNode.Text := 'True'
    else
      cNode.Text := 'False';
    pNode.ChildNodes.Add(cNode);

    //UndoRedoSize
    cNode := document.CreateNode('UndoRedoSize');
    cNode.Text := IntToStr(MyCAD.UndoRedoSize);
    pNode.ChildNodes.Add(cNode);

    //UnitType
    cNode := document.CreateNode('UnitType');
    cNode.Text := IntToStr(Ord(MyCAD.TheUNIT));
    pNode.ChildNodes.Add(cNode);

    //DiskFileVersion
    cNode := document.CreateNode('DiskFileVersion');
    cNode.Text := MyCAD.DiskFileVersion;
    pNode.ChildNodes.Add(cNode);

    //XYMode
    cNode := document.CreateNode('XYMode');
    cNode.Text := IntToStr(Ord(MyCAD.XYMode));
    pNode.ChildNodes.Add(cNode);

    //Zoom
    cNode := document.CreateNode('Zoom');
    cNode.Text := FloatToStr(MyCAD.Zoom);
    pNode.ChildNodes.Add(cNode);

     //Shapes
    cNode := document.CreateNode('Shapes');
    document.DocumentElement.ChildNodes.Add(cNode);

    for i:=0 to MyCAD.GetShapesCount()-1 do
    begin
      //Line
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyLine' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyLine');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
        SaveShapeLineToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
      end;
      //RuleLine
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyRuleLine' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyRuleLine');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
        SaveShapeLineToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
        SaveShapeRuleLineToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
      end;
      //LinkLine
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyLinkLine' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyLinkLine');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
        SaveShapeLineToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
      end;
      //PolyLine
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyPolyLine' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyPolyLine');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
        SaveShapeLineToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
      end;
      //FreeLine
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyFreeLine' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyFreeLine');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
        SaveShapeLineToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
      end;
      //Polygon
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyPolygon' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyPolygon');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
        SaveShapeLineToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
      end;

      //Rectangle
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyRectangle' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyRectangle');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);

      end;

      //Image
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyImage' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyImage');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
        SaveShapeImageToDoc(MyCAD.GetShapeByNo(i),document,cNode2);

      end;

      //Ellipse
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyEllipse' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyEllipse');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);

      end;

      //Text
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyText' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyText');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
        SaveShapeTextToDoc(MyCAD.GetShapeByNo(i),document,cNode2);

      end;

      //Group
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyGroup' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyGroup');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);

      end;

      //Combine
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyCombine' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyCombine');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);

      end;

      //LinkPoint
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyLinkPoint' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyLinkPoint');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
        SaveShapeLinkPointToDoc(MyCAD.GetShapeByNo(i),document,cNode2);

      end;

      //LineLinkLine
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyLineLinkLine' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyLineLinkLine');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
        SaveShapeLinkPointToDoc(MyCAD.GetShapeByNo(i),document,cNode2);

      end;

      //PolyBezier
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyPolyBezier' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyPolyBezier');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);

      end;

      //ElliArc
      if MyCAD.GetShapeByNo(i).ClassName = 'TMyElliArc' then
      begin
        cNode2 := document.CreateNode('Shape');
        cNode2.AttributeNodes.Add(document.CreateNode('ClassName',ntAttribute));
        cNode2.SetAttribute('ClassName','Codeidea.UltraGraphics.MyElliArc');
        cNode.ChildNodes.Add(cNode2);

        SaveShapeToDoc(MyCAD.GetShapeByNo(i),document,cNode2);
        SaveShapeElliArcToDoc(MyCAD.GetShapeByNo(i),document,cNode2);

      end;


    end;


    result := true;
  except

  end;
end;

procedure SaveShapeToDoc(MyShape: TMyShape;document: TXmlDocument; shape: IXMLNode);
var
  i:integer;
  cn,cn2:IXMLNode;
begin
  //CenterPoint
  cn := document.CreateNode('CenterPoint');
  cn.AttributeNodes.Add(document.CreateNode('X',ntAttribute));
  cn.AttributeNodes.Add(document.CreateNode('Y',ntAttribute));
  cn.SetAttribute('X',FloatToStr(MyShape.CenterPoint.X));
  cn.SetAttribute('Y',FloatToStr(MyShape.CenterPoint.Y));
  shape.ChildNodes.Add(cn);

  //ChildShapesNo
  cn := document.CreateNode('ChildShapesNo');
  shape.ChildNodes.Add(cn);
  for i:=0 to high(MyShape.ChildShapesNo) do
  begin
    cn2 := document.CreateNode('ChildShapeNo');
    cn2.AttributeNodes.Add(document.CreateNode('Value',ntAttribute));
    cn2.SetAttribute('Value',FloatToStr(MyShape.ChildShapesNo[i]));
    cn.ChildNodes.Add(cn2);
  end;

  //LayerId
	cn := document.CreateNode('LayerId');
  cn.Text := IntToStr(MyShape.LayerID);
  shape.ChildNodes.Add(cn);

  //LinkPoints
  cn := document.CreateNode('LinkPoints');
  shape.ChildNodes.Add(cn);
  for i:=0 to high(MyShape.LinkPoints) do
  begin
    cn2 := document.CreateNode('LinkPoint');
    cn2.AttributeNodes.Add(document.CreateNode('X',ntAttribute));
    cn2.AttributeNodes.Add(document.CreateNode('Y',ntAttribute));
    cn2.SetAttribute('X',FloatToStr(MyShape.LinkPoints[i].X));
    cn2.SetAttribute('Y',FloatToStr(MyShape.LinkPoints[i].Y));
    cn.ChildNodes.Add(cn2);
  end;

  //LinkShapesNo
  cn := document.CreateNode('LinkShapesNo');
  shape.ChildNodes.Add(cn);
  for i:=0 to high(MyShape.LinkShapesNo) do
  begin
    cn2 := document.CreateNode('LinkShapesNo');
    cn2.AttributeNodes.Add(document.CreateNode('Value',ntAttribute));
    cn2.SetAttribute('Value',IntToStr(MyShape.LinkshapesNo[i]));
    cn.ChildNodes.Add(cn2);
  end;

  //ParentShapeNo
	cn := document.CreateNode('ParentShapeNo');
  cn.Text := IntToStr(MyShape.ParentShapeNo);
  shape.ChildNodes.Add(cn);

  //SelectedHotId
	cn := document.CreateNode('SelectedHotId');
  cn.Text := IntToStr(MyShape.SelectedHotId);
  shape.ChildNodes.Add(cn);

  //ShapeId
	cn := document.CreateNode('ShapeId');
  cn.Text := IntToStr(MyShape.ShapeId);
  shape.ChildNodes.Add(cn);

  //ShapeNo
	cn := document.CreateNode('ShapeNo');
  cn.Text := IntToStr(MyShape.ShapeNo);
  shape.ChildNodes.Add(cn);

  //TextOutPoint
	cn := document.CreateNode('TextOutPoint');
  cn.AttributeNodes.Add(document.CreateNode('X',ntAttribute));
  cn.AttributeNodes.Add(document.CreateNode('Y',ntAttribute));
  cn.SetAttribute('X',FloatToStr(MyShape.TextOutPoint.X));
  cn.SetAttribute('Y',FloatToStr(MyShape.TextOutPoint.Y));
  shape.ChildNodes.Add(cn);

  //ThePoints
  cn := document.CreateNode('ThePoints');
  shape.ChildNodes.Add(cn);
  for i:=0 to high(MyShape.ThePoints) do
  begin
    cn2 := document.CreateNode('ThePoint');
    cn2.AttributeNodes.Add(document.CreateNode('X',ntAttribute));
    cn2.AttributeNodes.Add(document.CreateNode('Y',ntAttribute));
    cn2.SetAttribute('X',FloatToStr(MyShape.ThePoints[i].X));
    cn2.SetAttribute('Y',FloatToStr(MyShape.ThePoints[i].Y));
    cn.ChildNodes.Add(cn2);
  end;

  {}
  //Alpha
  {
  cn := document.CreateNode('Alpha');
  shape.ChildNodes.Add(cn);
  }

  //Angle
  cn := document.CreateNode('Angle');
  cn.Text := FloatToStr(MyShape.Angle);
  shape.ChildNodes.Add(cn);

  //Caption
  cn := document.CreateNode('Caption');
  cn.Text := MyShape.Caption;
  shape.ChildNodes.Add(cn);

  //CaptionShow
  cn := document.CreateNode('CaptionShow');
  if MyShape.CaptionShow then
    cn.Text := 'True'
  else
    cn.Text := 'False';
  shape.ChildNodes.Add(cn);

  //IsFlipHorizontal
  cn := document.CreateNode('IsFlipHorizontal');
  if MyShape.IsFlipHorz then
    cn.Text := 'True'
  else
    cn.Text := 'False';
  shape.ChildNodes.Add(cn);

  //IsFlipVertical
  cn := document.CreateNode('IsFlipVertical');
  if MyShape.IsFlipVert then
    cn.Text := 'True'
  else
    cn.Text := 'False';
  shape.ChildNodes.Add(cn);

  //Info
  cn := document.CreateNode('Info');
  if length(MyShape.Info)>0 then
    cn.Text := MyShape.Info;
  shape.ChildNodes.Add(cn);

  //Lock
  cn := document.CreateNode('Lock');
  if MyShape.Locked then
    cn.Text := 'True'
  else
    cn.Text := 'False';
  shape.ChildNodes.Add(cn);

  //Name
  cn := document.CreateNode('Name');
  cn.Text := MyShape.Name;
  shape.ChildNodes.Add(cn);

  //ShowUnit - .net
  {
  cn := document.CreateNode('ShowUnit');
  shape.ChildNodes.Add(cn);
  }

  //Tag
  cn := document.CreateNode('Tag');
  cn.Text := IntToStr(MyShape.Tag);
  shape.ChildNodes.Add(cn);

  //UserData
  cn := document.CreateNode('UserDatas');
  shape.ChildNodes.Add(cn);
  for i:=0 to MyShape.UserData.GetCount-1 do
  begin
    cn2 := document.CreateNode('UserData');
    cn2.AttributeNodes.Add(document.CreateNode('Key',ntAttribute));
    cn2.AttributeNodes.Add(document.CreateNode('Value',ntAttribute));
    cn2.SetAttribute('Key',MyShape.UserData.UserDataRecord[i].Key);
    cn2.SetAttribute('Value',MyShape.UserData.UserDataRecord[i].Value);
    cn.ChildNodes.Add(cn2);
  end;

  //Visible
  cn := document.CreateNode('Visible');
  if MyShape.Visible then
    cn.Text := 'True'
  else
    cn.Text := 'False';
  shape.ChildNodes.Add(cn);



end;

procedure SaveShapeLineToDoc(MyShape:TMyShape;document:TXmlDocument;shape:IXMLNode);
var
  cn:IXMLNode;
begin

  //ArrowAngle
	cn := document.CreateNode('ArrowAngle');
  cn.Text := IntToStr((MyShape as TMyLine).ArrowAngle);
  shape.ChildNodes.Add(cn);

  //ArrowLength
	cn := document.CreateNode('ArrowLength');
  cn.Text := IntToStr((MyShape as TMyLine).ArrowLength);
  shape.ChildNodes.Add(cn);

  //ArrowOffset
	cn := document.CreateNode('ArrowOffset');
  cn.Text := IntToStr((MyShape as TMyLine).ArrowOffset);
  shape.ChildNodes.Add(cn);

  //ArrowStyle
	cn := document.CreateNode('ArrowStyle');
  cn.Text := IntToStr(Ord((MyShape as TMyLine).ArrowStyle));
  shape.ChildNodes.Add(cn);
end;

procedure SaveShapeRuleLineToDoc(MyShape:TMyShape;document:TXmlDocument;shape:IXMLNode);
var
  cn:IXMLNode;
begin

  //ShowUserInfo
	cn := document.CreateNode('ShowUserInfo');
  if (MyShape as TMyRuleLine).ShowUserInfo then
    cn.Text := 'True'
  else
    cn.Text := 'False';
  shape.ChildNodes.Add(cn);

  //TickStyle
	cn := document.CreateNode('TickStyle');
  cn.Text := IntToStr(ord((MyShape as TMyRuleLine).TickStyle));
  shape.ChildNodes.Add(cn);

  //UserInfo
	cn := document.CreateNode('UserInfo');
  cn.Text := (MyShape as TMyRuleLine).UserInfo;
  shape.ChildNodes.Add(cn);
end;

procedure SaveShapeImageToDoc(MyShape:TMyShape;document:TXmlDocument;shape:IXMLNode);
var

  cn:IXMLNode;

  mem:TMemoryStream;
  strStream:TStringStream;
begin

  //BackgroundBitmap
  if Assigned((MyShape as TMyImage).Bitmap) and
    ((MyShape as TMyImage).Bitmap.Width >0) and
    ((MyShape as TMyImage).Bitmap.Height >0) then
  begin

    strStream := TStringStream.Create('');
    mem := TMemoryStream.Create;
    (MyShape as TMyImage).Bitmap.SaveToStream(mem);
    mem.Position := 0;
    EncodeStream(mem,strStream);

    cn := document.CreateNode('Bitmap');
    cn.Text := strStream.DataString;
    shape.ChildNodes.Add(cn);

    strStream.Destroy;
    mem.Destroy;

  end;

  //Border
  cn := document.CreateNode('Border');
  if (MyShape as TMyImage).Border then
    cn.Text := 'True'
  else
    cn.Text := 'False';
  shape.ChildNodes.Add(cn);

  //Transparent
  cn := document.CreateNode('Transparent');
  if (MyShape as TMyImage).Transparent then
    cn.Text := 'True'
  else
    cn.Text := 'False';
  shape.ChildNodes.Add(cn);
end;

procedure SaveShapeTextToDoc(MyShape:TMyShape;document:TXmlDocument;shape:IXMLNode);
var
  cn,cn2:IXMLNode;
  i:integer;
begin

  //Border
	cn := document.CreateNode('Border');
  if (MyShape as TMyText).IsBorder then
    cn.Text := 'True'
  else
    cn.Text := 'False';
  shape.ChildNodes.Add(cn);
  //HAlignment
	cn := document.CreateNode('HAlignment');
  cn.Text := IntToStr(Ord((MyShape as TMyText).HAlignment));
  shape.ChildNodes.Add(cn);
  //Lines
  cn := document.CreateNode('Lines');
  shape.ChildNodes.Add(cn);
  for i:=0 to (MyShape as TMyText).Lines.Count-1 do
  begin
    cn2 := document.CreateNode('Line');
    cn2.Text := (MyShape as TMyText).Lines[i];
    cn.ChildNodes.Add(cn2);
  end;
  //VAlignment
	cn := document.CreateNode('VAlignment');
  cn.Text := IntToStr(Ord((MyShape as TMyText).VAlignment));
  shape.ChildNodes.Add(cn);
  //WordWrap
	cn := document.CreateNode('WordWrap');
  if (MyShape as TMyText).WordWrap then
    cn.Text := 'True'
  else
    cn.Text := 'False';
  shape.ChildNodes.Add(cn);
end;

procedure SaveShapeLinkPointToDoc(MyShape:TMyShape;document:TXmlDocument;shape:IXMLNode);
var
  cn:IXMLNode;
begin
  //Size
	cn := document.CreateNode('Size');
  cn.Text := IntToStr((MyShape as TMyLinkPoint).Size);
  shape.ChildNodes.Add(cn);
end;

procedure SaveShapeElliArcToDoc(MyShape:TMyShape;document:TXmlDocument;shape:IXMLNode);
var
  cn:IXMLNode;
begin

  //ArcMode
	cn := document.CreateNode('ArcMode');
  cn.Text := IntToStr(Ord((MyShape as TMyElliArc).ArcMode));
  shape.ChildNodes.Add(cn);

  //ArcStyle
	cn := document.CreateNode('ArcStyle');
  cn.Text := IntToStr(Ord((MyShape as TMyElliArc).ArcStyle));
  shape.ChildNodes.Add(cn);
end;

end.
